# A common place to put backports for specific versions of Python
# Currently empty, but see example below

# import sys
# if sys.version_info >= (3, 11):
#     from typing import LiteralString
# else:
#     # https://pypi.org/project/typing-extensions/
#     from typing_extensions import LiteralString
